@@ -2,6 +2,7 @@ mod dataset;
 mod source;
 
 use self::{dataset::UpdateDatasetArgs, source::UpdateSourceArgs};
+use crate::printer::Printer;
 use anyhow::Result;
 use reinfer_client::Client;
 use structopt::StructOpt;
@@ -17,9 +18,9 @@ pub enum UpdateArgs {
     Dataset(UpdateDatasetArgs),
 }
 
-pub fn run(update_args: &UpdateArgs, client: Client) -> Result<()> {
+pub fn run(update_args: &UpdateArgs, client: Client, printer: &Printer) -> Result<()> {
     match update_args {
-        UpdateArgs::Source(source_args) => source::update(&client, source_args),
-        UpdateArgs::Dataset(dataset_args) => dataset::update(&client, dataset_args),
+        UpdateArgs::Source(source_args) => source::update(&client, source_args, printer),
+        UpdateArgs::Dataset(dataset_args) => dataset::update(&client, dataset_args, printer),
     }
 }
